/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.base.extensions

import com.alibaba.fastjson.PropertyNamingStrategy
import com.persagy.base.utils.SJsonUtil
import java.lang.Exception
import java.lang.reflect.ParameterizedType

/**
 * Any扩展。获得类的泛型类
 *
 * @return  类的泛型类
 */
fun Any.gennericClass(): Class<*>? {
    return try {
        (javaClass.genericSuperclass as ParameterizedType).actualTypeArguments[0] as Class<*>
    } catch (e: Exception) {
        null
    }
} // Fun Any.gennericClass()

/**
 * Any扩展toJson()函数。
 *
 * @param   namingStrategy          命名规则
 * @return  Json字符串(属性为null不序列化)
 */
fun Any.toJson(namingStrategy: PropertyNamingStrategy? = null): String {
    return SJsonUtil.toJson(this, namingStrategy)
} // Fun Any.toJson()

/**
 * Any扩展toJson()函数。
 *
 * @param   namingStrategy          命名规则
 * @return  Json字符串(属性为null序列化)
 */
fun Any.toJsonAll(namingStrategy: PropertyNamingStrategy? = null): String {
    return SJsonUtil.toJsonAll(this, namingStrategy)
} // Fun Any.toJsonAll()

/**
 * Any扩展toMap()函数。
 *
 * @return  Json字符串
 */
fun Any.toMap(): Map<String, Any?> {
    @Suppress("UNCHECKED_CAST")
    return SJsonUtil.fromJson(this.toJson(), Map::class.java) as Map<String, Any?>
} // Fun Any.toMap()

/**
 * Any扩展toByte()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toBoolean(): Boolean? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> this as Boolean
        "Byte" -> (this as Byte).toBoolean()
        "Double" -> (this as Double).toBoolean()
        "Float" -> (this as Float).toBoolean()
        "Int", "Integer" ->  (this as Int).toBoolean()
        "Long" -> (this as Long).toBoolean()
        "Short" -> (this as Short).toBoolean()
        else -> null
    }
} // Fun Any.toBoolean()

/**
 * Any扩展toByte()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toByte(): Byte? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> (this as Boolean).toByte()
        "Byte" -> this as Byte
        "Double" -> (this as Double).toByte()
        "Float" -> (this as Float).toByte()
        "Int", "Integer" ->  (this as Int).toByte()
        "Long" -> (this as Long).toByte()
        "Short" -> (this as Short).toByte()
        else -> null
    }
} // Fun Any.toByte()

/**
 * Any扩展toDouble()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toDouble(): Double? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> (this as Boolean).toDouble()
        "Byte" -> (this as Byte).toDouble()
        "Double" -> this as Double
        "Float" -> (this as Float).toDouble()
        "Int", "Integer" ->  (this as Int).toDouble()
        "Long" -> (this as Long).toDouble()
        "Short" -> (this as Short).toDouble()
        else -> null
    }
} // Fun Any.toDouble()

/**
 * Any扩展toFloat()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toFloat(): Float? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> (this as Boolean).toFloat()
        "Byte" -> (this as Byte).toFloat()
        "Double" -> (this as Double).toFloat()
        "Float" -> this as Float
        "Int", "Integer" ->  (this as Int).toFloat()
        "Long" -> (this as Long).toFloat()
        "Short" -> (this as Short).toFloat()
        else -> null
    }
} // Fun Any.toFloat()

/**
 * Any扩展toInt()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toInt(): Int? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> (this as Boolean).toInt()
        "Byte" -> (this as Byte).toInt()
        "Double" -> (this as Double).toInt()
        "Float" -> (this as Float).toInt()
        "Int", "Integer" ->  this as Int
        "Long" -> (this as Long).toInt()
        "Short" -> (this as Short).toInt()
        "String" -> {
            try {
                Integer.parseInt(this as String)
            }catch (e: Exception) {
                0
            }
        }
        else -> null
    }
} // Fun Any.toInt()

/**
 * Any扩展toLong()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toLong(): Long? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> (this as Boolean).toLong()
        "Byte" -> (this as Byte).toLong()
        "Double" -> (this as Double).toLong()
        "Float" -> (this as Float).toLong()
        "Int", "Integer" ->  (this as Int).toLong()
        "Long" -> this as Long
        "Short" -> (this as Short).toLong()
        else -> null
    }
} // Fun Any.toLong()

/**
 * Any扩展toShort()函数。
 *
 * @return  将对象转换为Int类型
 */
fun Any.toShort(): Short? {
    return when(this.javaClass.simpleName) {
        "Boolean" -> (this as Boolean).toShort()
        "Byte" -> (this as Byte).toShort()
        "Double" -> (this as Double). toShort()
        "Float" -> (this as Float).toShort()
        "Int", "Integer" ->  (this as Int).toShort()
        "Long" -> (this as Long).toShort()
        "Short" -> this as Short
        else -> null
    }
} // Fun Any.toShort()

/**
 * Any扩展，判断是否为基本类型
 *
 * @return  如果是则返回true
 */
fun Any.isBaseType(): Boolean {
    if (this.javaClass.isPrimitive) {
        return true;
    }

    return  this is String ||
            this is Boolean ||
            this is Char ||
            this is Byte ||
            this is Short ||
            this is Int ||
            this is Long ||
            this is Float ||
            this is Double ||
            this is java.sql.Date ||
            this is java.sql.Time ||
            this is java.sql.Timestamp ||
            this is java.util.Date
} // Fun Any.isBaseType()
