/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.base.extensions

import com.alibaba.fastjson.annotation.JSONField
import java.lang.reflect.Field
import java.lang.reflect.Method

/**
 * Class<*>扩展。获得所有包含父对象的定义字段列表。
 *
 * @return  字段列表
 */
fun Class<*>.getAllDeclaredFields(): ArrayList<Field> {
    val fieldList = ArrayList<Field>()
    var clz: Class<*>? = this
    while (clz != null) {
        fieldList.addAll(clz.declaredFields.toList())
        clz = clz.superclass
    }
    return fieldList
} // Fun Class<*>.getAllDeclaredFields()

/**
 * Class<*>扩展。根据名称获得所有包含父对象的定义字段。
 *
 * @param   ignoreCase      是否忽略大小写，默认不忽略
 * @param   jsonProperty    是否检测Json属性
 * @return  字段
 */
fun Class<*>.getAllDeclaredField(name: String, ignoreCase: Boolean = false, jsonProperty: Boolean = false): Field? {
    val list = this.getAllDeclaredFields()
    return list.first {
        val annName = it.getAnnotation(JSONField::class.java)?.name
        it.name.equals(name, ignoreCase) || (jsonProperty && name.equals(annName, ignoreCase))
    }
} // Fun Class<*>.getAllDeclaredField()

/**
 * Class<*>扩展。获得所有包含父对象的定义方法列表。
 *
 * @return  方法列表
 */
fun Class<*>.getAllDeclaredMethods(): ArrayList<Method> {
    val methodList = ArrayList<Method>()
    var clz: Class<*>? = this
    while (clz != null) {
        methodList.addAll(clz.declaredMethods.toList())
        clz = clz.superclass
    }
    return methodList
} // Fun Class<*>.getAllDeclaredMethods()

/**
 * Class<*>扩展，判断是否为基本类型
 *
 * @return  如果是则返回true
 */
fun Class<*>.isBaseType(): Boolean {
    if (this.isPrimitive) {
        return true;
    }

    return  String::class.java.isAssignableFrom(this) ||
            Boolean::class.java.isAssignableFrom(this) ||
            Char::class.java.isAssignableFrom(this) ||
            Byte::class.java.isAssignableFrom(this) ||
            Short::class.java.isAssignableFrom(this) ||
            Int::class.java.isAssignableFrom(this) ||
            Long::class.java.isAssignableFrom(this) ||
            Float::class.java.isAssignableFrom(this) ||
            Double::class.java.isAssignableFrom(this) ||
            java.sql.Date::class.java.isAssignableFrom(this) ||
            java.sql.Time::class.java.isAssignableFrom(this) ||
            java.sql.Timestamp::class.java.isAssignableFrom(this) ||
            java.util.Date::class.java.isAssignableFrom(this)

} // Fun Class<*>.isBaseType()