/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.base.utils

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

/**
 * Gzip工具，用于完成zip压缩与解压缩
 *
 * @author  庞利祥 <sybotan@126.com>
 */
object SGzipUtil {
    const val GZIP_ENCODE_UTF_8 = "UTF-8"
    const val GZIP_ENCODE_ISO_8859_1 = "ISO-8859-1"

    /**
     * 字符串的解压
     *
     * @param   str         对字符串解压
     * @return  返回解压缩后的字符串
     */
    @Throws(IOException::class)
    fun unCompress(str: String): ByteArray? {
        if (str.isEmpty()) {
            return null
        }

        // 创建一个新的 byte 数组输出流
        val out = ByteArrayOutputStream()
        // 创建一个 ByteArrayInputStream，使用 buf 作为其缓冲区数组
        val `in` = ByteArrayInputStream(str
                .toByteArray(charset("UTF-8")))
        // 使用默认缓冲区大小创建新的输入流
        val gzip = GZIPInputStream(`in`)
        val buffer = ByteArray(1024)
        var n = gzip.read(buffer)
        while (n >= 0) {  // 将未压缩数据读入字节数组
            // 将指定 byte 数组中从偏移量 off 开始的 len 个字节写入此 byte数组输出流
            out.write(buffer, 0, n)
            n = gzip.read(buffer)
        }
        // 使用指定的 charsetName，通过解码字节将缓冲区内容转换为字符串
        return buffer
    } // Fun unCompress()

    /**
     * 字符串压缩为GZIP字节数组
     */
    fun compress(str: String, encoding: String): ByteArray? {
        if (str.isEmpty()) {
            return null
        }

        val out = ByteArrayOutputStream()
        val gzip: GZIPOutputStream
        try {
            gzip = GZIPOutputStream(out)
            gzip.write(str.toByteArray(charset(encoding)))
            gzip.close()
        } catch (e: IOException) {
            println("gzip compress error: " + e.message)
        }

        return out.toByteArray()
    } // Fun compress()

    /**
     * Gzip  byte[] 解压成字符串
     */
    fun uncompressToString(bytes: ByteArray): String? {
        return uncompressToString(bytes, GZIP_ENCODE_UTF_8)
    } // Fun uncompressToString()


    /**
     * Gzip  byte[] 解压成字符串
     */
    private fun uncompressToString(bytes: ByteArray?, encoding: String): String? {
        if (bytes == null || bytes.size == 0) {
            return null
        }
        val out = ByteArrayOutputStream()
        val `in` = ByteArrayInputStream(bytes)
        try {
            val ungzip = GZIPInputStream(`in`)
            val buffer = ByteArray(1024)
            var n = ungzip.read(buffer)
            while (n >= 0) {
                out.write(buffer, 0, n)
                n = ungzip.read(buffer)
            }
            return out.toString(encoding)
        } catch (e: IOException) {
            println("gzip compress error:" + e.message)
        }

        return null
    } // Fun uncompressToString()

//    /**
//     * 判断byte[]是否是Gzip格式
//     */
//    fun isGzip(data: ByteArray): Boolean {
//        val header = data[0] shl 8 or (data[1] and 0xff)
//        return header == 0x1f8b
//    } // Fun isGzip()

//    /**
//     * GZIP解压缩
//     *
//     * @param   zipText     GZIP压缩过的字符串
//     * @return 解压后的字符串
//     */
//    @Throws(IOException::class)
//    fun decompress(zipText: String): String {
//        val compressed = Base64.decode(zipText, 1024)
//        if (compressed.size > 4) {
//            val gzipInputStream = GZIPInputStream(
//                    ByteArrayInputStream(compressed, 4,
//                            compressed.size - 4))
//
//            val baos = ByteArrayOutputStream()
//            var value = 0
//            while (value != -1) {
//                value = gzipInputStream.read()
//                if (value != -1) {
//                    baos.write(value)
//                }
//            }
//            gzipInputStream.close()
//            baos.close()
//            return String(baos.toByteArray(), "UTF-8")
//        } else {
//            return ""
//        }
//    }
} // Object GzipUtil