/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.database

import com.persagy.database.enums.SOps

/**
 * 过滤条件
 *
 * @author  庞利祥 <sybotan@126.com>
 */
class SFilter private constructor(
        /** 第一参数，保存属性名 */
        var first: Any?,
        /** 第一参数，保存值 */
        var second: Any?,
        /** 操作符 */
        var opt: SOps = SOps.EQ) {
    constructor(): this(null, null)

    companion object {
        /**
         * 过滤条件：属性 == 值
         *
         * @param   property        属性名
         * @param   value           值
         * @return  过滤条件
         */
        fun eq(property: String, value: Any): SFilter {
            return SFilter(property, value, SOps.EQ)
        } // Fun eq()

        /**
         * 过滤条件：函数 == 值
         *
         * @param   func            函数对象
         * @param   value           值
         * @return  过滤条件
         */
        fun eq(func: SAbstractFunction, value: Any): SFilter {
            return SFilter(func, value, SOps.EQ)
        } // Fun eq()

        /**
         * 过滤条件：属性 != 值
         *
         * @param   property        属性名
         * @param   value           值
         * @return  过滤条件
         */
        fun ne(property: String, value: Any): SFilter {
            return SFilter(property, value, SOps.NE)
        } // Fun ne()

        /**
         * 过滤条件：函数 != 值
         *
         * @param   func            函数对象
         * @param   value           值
         * @return  过滤条件
         */
        fun ne(func: SAbstractFunction, value: Any): SFilter {
            return SFilter(func, value, SOps.NE)
        } // Fun ne()

        /**
         * 过滤条件：属性 > 值
         *
         * @param   property        属性名
         * @param   value           值
         * @return  过滤条件
         */
        fun gt(property: String, value: Any): SFilter {
            return SFilter(property, value, SOps.GT)
        } // Fun gt()

        /**
         * 过滤条件：函数 > 值
         *
         * @param   func            函数对象
         * @param   value           值
         * @return  过滤条件
         */
        fun gt(func: SAbstractFunction, value: Any): SFilter {
            return SFilter(func, value, SOps.GT)
        } // Fun gt()

        /**
         * 过滤条件：属性 >= 值
         *
         * @param   property        属性名
         * @param   value           值
         * @return  过滤条件
         */
        fun gte(property: String, value: Any): SFilter {
            return SFilter(property, value, SOps.GTE)
        } // Fun gte()

        /**
         * 过滤条件：函数 >= 值
         *
         * @param   func            函数对象
         * @param   value           值
         * @return  过滤条件
         */
        fun gte(func: SAbstractFunction, value: Any): SFilter {
            return SFilter(func, value, SOps.GTE)
        } // Fun gte()

        /**
         * 过滤条件：属性 < 值
         *
         * @param   property        属性名
         * @param   value           值
         * @return  过滤条件
         */
        fun lt(property: String, value: Any): SFilter {
            return SFilter(property, value, SOps.LT)
        } // Fun lt()

        /**
         * 过滤条件：函数 < 值
         *
         * @param   func            函数对象
         * @param   value           值
         * @return  过滤条件
         */
        fun lt(func: SAbstractFunction, value: Any): SFilter {
            return SFilter(func, value, SOps.LT)
        } // Fun lt()

        /**
         * 过滤条件：属性 <= 值
         *
         * @param   property        属性名
         * @param   value           值
         * @return  过滤条件
         */
        fun lte(property: String, value: Any): SFilter {
            return SFilter(property, value, SOps.LTE)
        } // Fun lte()

        /**
         * 过滤条件：函数 <= 值
         *
         * @param   func            函数对象
         * @param   value           值
         * @return  过滤条件
         */
        fun lte(func: SAbstractFunction, value: Any): SFilter {
            return SFilter(func, value, SOps.LTE)
        } // Fun lte()

        /**
         * 过滤条件：包含
         *
         * @param   property        属性名
         * @param   value           被包含的值
         * @return  过滤条件
         */
        fun contain(property: String, value: String): SFilter {
            return SFilter(property, value, SOps.CONTAIN)
        } // Fun contain()

        /**
         * 过滤条件：包含
         *
         * @param   func            函数对象
         * @param   value           被包含的值
         * @return  过滤条件
         */
        fun contain(func: SAbstractFunction, value: String): SFilter {
            return SFilter(func, value, SOps.CONTAIN)
        } // Fun contain()

        /**
         * 过滤条件：以指定字符串开始
         *
         * @param   property        属性名
         * @param   value           指定字符串开始
         * @return  过滤条件
         */
        fun startWith(property: String, value: String): SFilter {
            return SFilter(property, value, SOps.STARTWITH)
        } // Fun contain()

        /**
         * 过滤条件：以指定字符串开始
         *
         * @param   func            函数对象
         * @param   value           指定字符串开始
         * @return  过滤条件
         */
        fun startWith(func: SAbstractFunction, value: String): SFilter {
            return SFilter(func, value, SOps.STARTWITH)
        } // Fun contain()

        /**
         * 过滤条件：以指定字符串结尾
         *
         * @param   property        属性名
         * @param   value           指定字符串开始
         * @return  过滤条件
         */
        fun endWith(property: String, value: String): SFilter {
            return SFilter(property, value, SOps.ENDWITH)
        } // Fun endWith()

        /**
         * 过滤条件：以指定字符串结尾
         *
         * @param   func            函数对象
         * @param   value           指定字符串开始
         * @return  过滤条件
         */
        fun endWith(func: SAbstractFunction, value: String): SFilter {
            return SFilter(func, value, SOps.ENDWITH)
        } // Fun endWith()

        /**
         * 过滤条件：属性为空
         *
         * @param   property        属性名
         * @return  过滤条件
         */
        fun isNull(property: String): SFilter {
            return SFilter(property, null, SOps.ISNULL)
        } // Fun isNull()

        /**
         * 过滤条件：属性为空
         *
         * @param   func            函数对象
         * @return  过滤条件
         */
        fun isNull(func: SAbstractFunction): SFilter {
            return SFilter(func, null, SOps.ISNULL)
        } // Fun isNull()

        /**
         * 过滤条件：属性在值列表中
         *
         * @param   property        属性名
         * @param   valueList       值列表
         * @return  过滤条件
         */
        fun inList(property: String, valueList: List<Any>): SFilter {
            return SFilter(property, valueList, SOps.IN)
        } // Fun inList()

        /**
         * 过滤条件：属性在值列表中
         *
         * @param   func            函数对象
         * @param   valueList       值列表
         * @return  过滤条件
         */
        fun inList(func: SAbstractFunction, valueList: List<Any>): SFilter {
            return SFilter(func, valueList, SOps.IN)
        } // Fun inList()

        /**
         * 过滤条件：逻辑非
         *
         * @param   filter          属性名
         * @return  过滤条件
         */
        fun not(filter: SFilter): SFilter {
            return SFilter(filter, null, SOps.NOT)
        } // Fun not()

        /**
         * 过滤条件：逻辑与
         *
         * @param   filterList      过滤条件列表
         * @return  过滤条件
         */
        fun and(vararg filterList: SFilter): SFilter {
            var filter = filterList[0]
            for (i in 1 until filterList.size) {
                filter = SFilter(filter, filterList[i], SOps.AND)
            }
            return filter
        } // Fun not()

        /**
         * 过滤条件：逻辑或
         *
         * @param   filterList      过滤条件列表
         * @return  过滤条件
         */
        fun or(vararg filterList: SFilter): SFilter {
            var filter = filterList[0]
            for (i in 1 until filterList.size) {
                filter = SFilter(filter, filterList[i], SOps.OR)
            }
            return filter
        } // Fun not()
    } // companion object
} // Class SFilter