/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.database.extensions

import com.persagy.base.extensions.getAllDeclaredFields
import com.persagy.database.SFilter
import javax.persistence.Id

/**
 * 取得主键字段值
 *
 * @return  Id字段值
 */
fun Any.keyValue(property: String = "id"): Any? {
    /** 依次取id列表中的键值 */
    javaClass.keyFieldList().forEach {
        if (it.name == property) {
            val accessible = it.isAccessible   //canAccess(this)
            it.isAccessible = true
            val id = it.get(this)
            it.isAccessible = accessible
            return id
        }
    }
    return null
} // Fun Serializable.keyValue()

/**
 * 取得主键与值的映射
 *
 * @return  主键映射。key为属性名
 */
fun Any.keyMap(): HashMap<String, Any> {
    val map = HashMap<String, Any>()
    val clz = this::class.java
    for (field in clz.getAllDeclaredFields()) {
        // val anno = field.getAnnotation(Column::class.java)
        if (field.getAnnotation(Id::class.java) != null)
        {
            val accessible = field.isAccessible  //canAccess(this)
            field.isAccessible = true
            map[field.name] = field.get(this)
            field.isAccessible = accessible
        }
    }

    return map
} // Fun Serializable.keyMap()

/**
 * 获得主键的过滤条件
 *
 * @return  主键的过滤条件
 */
fun Any.filterListByKeys(): ArrayList<SFilter> {
    val filterList = ArrayList<SFilter>()
    this.keyMap().forEach {
        filterList.add(SFilter.eq(it.key, it.value))
    }
    return filterList
} // Fun Serializable.filterListByKeys()