/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.database.test.parser

import org.testng.annotations.DataProvider
import org.testng.annotations.Test

/**
 * 语法解析器-测试类
 *
 * @author  庞利祥 <sybotan@126.com>
 */
class SSyntaxParserTest {

    /**
     * 提供正确过滤条件测试数据
     *
     * @param   filter      过滤条件
     * @param   result      预期结果
     */
    @DataProvider(name="filter")
    fun goodFilterDataProvider(): Array<Array<String>> {
        return arrayOf(
                // 关系
                arrayOf("id = 1", "[{\"first\":\"id\",\"second\":1,\"opt\":\"EQ\"}]"),
                arrayOf("id = '2'", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"EQ\"}]"),
                arrayOf("id > '2'", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"GT\"}]"),
                arrayOf(" id >= '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"GTE\"}]"),
                arrayOf(" id < '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"LT\"}]"),
                arrayOf(" id <= '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"LTE\"}]"),
                arrayOf(" id != '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"NE\"}]"),
                // 包含
                arrayOf(" id CONTAIN '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"CONTAIN\"}]"),
                arrayOf(" id contain '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"CONTAIN\"}]"),
                arrayOf(" id ContaiN '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"CONTAIN\"}]"),
                arrayOf(" id CoNTaIN '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"CONTAIN\"}]"),
                // 以XXX开始
                arrayOf(" id STARTWITH '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"STARTWITH\"}]"),
                arrayOf(" id startwith '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"STARTWITH\"}]"),
                arrayOf(" id STarTWiTH '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"STARTWITH\"}]"),
                // 以XXX结束
                arrayOf(" id ENDWITH '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"ENDWITH\"}]"),
                arrayOf(" id endwith '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"ENDWITH\"}]"),
                arrayOf(" id EndWiTH '2' ", "[{\"first\":\"id\",\"second\":\"2\",\"opt\":\"ENDWITH\"}]"),
                // 为空
                arrayOf(" id ISNULL ", "[{\"first\":\"id\",\"opt\":\"ISNULL\"}]"),
                arrayOf(" id isnull ", "[{\"first\":\"id\",\"opt\":\"ISNULL\"}]"),
                arrayOf(" id IsNull ", "[{\"first\":\"id\",\"opt\":\"ISNULL\"}]"),
                // 不为空
                arrayOf(" id NOTNULL ", "[{\"first\":\"id\",\"opt\":\"NOTNULL\"}]"),
                arrayOf(" id notnull ", "[{\"first\":\"id\",\"opt\":\"NOTNULL\"}]"),
                arrayOf(" id NotNULL ", "[{\"first\":\"id\",\"opt\":\"NOTNULL\"}]"),
                // 数据类型
                arrayOf(" used = true", "[{\"first\":\"used\",\"second\":true,\"opt\":\"EQ\"}]"),
                arrayOf(" used != TRUE", "[{\"first\":\"used\",\"second\":true,\"opt\":\"NE\"}]"),
                arrayOf(" used = TruE", "[{\"first\":\"used\",\"second\":true,\"opt\":\"EQ\"}]"),
                arrayOf(" used = false", "[{\"first\":\"used\",\"second\":false,\"opt\":\"EQ\"}]"),
                arrayOf(" used != FALSE", "[{\"first\":\"used\",\"second\":false,\"opt\":\"NE\"}]"),
                arrayOf(" used = FaLse", "[{\"first\":\"used\",\"second\":false,\"opt\":\"EQ\"}]"),
                arrayOf(" age = 18", "[{\"first\":\"age\",\"second\":18,\"opt\":\"EQ\"}]"),
                arrayOf(" width = 18.15", "[{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"}]"),
                // 括号表达式
                arrayOf(" (  width = 18.15 )", "[{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"}]"),
                // NOT
                arrayOf(" not width = 18.15", "[{\"first\":{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"},\"opt\":\"NOT\"}]"),
                arrayOf(" NOT width = 18.15", "[{\"first\":{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"},\"opt\":\"NOT\"}]"),
                arrayOf(" Not width = 18.15", "[{\"first\":{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"},\"opt\":\"NOT\"}]"),
                arrayOf(" !width = 18.15 ", "[{\"first\":{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"},\"opt\":\"NOT\"}]"),
                arrayOf(" !(width = 18.15)", "[{\"first\":{\"first\":\"width\",\"second\":18.15,\"opt\":\"EQ\"},\"opt\":\"NOT\"}]"),
                // OR
                arrayOf(" (id = '1') or (width>=2)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"OR\"}]"),
                arrayOf(" (id = '1') OR (width>=2)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"OR\"}]"),
                arrayOf(" (id = '1') || (width>=2)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"OR\"}]"),
                arrayOf(" (id = '1') or (width>=2) || height > 3", "[{\"first\":{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"OR\"},\"second\":{\"first\":\"height\",\"second\":3,\"opt\":\"GT\"},\"opt\":\"OR\"}]"),
                // AND
                arrayOf(" (id = '1') and (width>=2)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"AND\"}]"),
                arrayOf(" (id = '1') AND (width>=2)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"AND\"}]"),
                arrayOf(" (id = '1') && (width>=2)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"AND\"}]"),
                arrayOf(" (id = '1') anD (width>=2) && height > 3", "[{\"first\":{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"AND\"},\"second\":{\"first\":\"height\",\"second\":3,\"opt\":\"GT\"},\"opt\":\"AND\"}]"),
                // 逻辑混合
                arrayOf(" (id = '1') and (width>=2) || height > 3", "[{\"first\":{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"opt\":\"AND\"},\"second\":{\"first\":\"height\",\"second\":3,\"opt\":\"GT\"},\"opt\":\"OR\"}]"),
                arrayOf(" (id = '1') || (width>=2) && height > 3", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"second\":{\"first\":\"height\",\"second\":3,\"opt\":\"GT\"},\"opt\":\"AND\"},\"opt\":\"OR\"}]"),
                arrayOf(" (id = '1') and (width>=2 || height > 3)", "[{\"first\":{\"first\":\"id\",\"second\":\"1\",\"opt\":\"EQ\"},\"second\":{\"first\":{\"first\":\"width\",\"second\":2,\"opt\":\"GTE\"},\"second\":{\"first\":\"height\",\"second\":3,\"opt\":\"GT\"},\"opt\":\"OR\"},\"opt\":\"AND\"}]"),
                // 条件列表
                arrayOf(" width <= 18.15; age>18; name='abc'", "[{\"first\":\"width\",\"second\":18.15,\"opt\":\"LTE\"},{\"first\":\"age\",\"second\":18,\"opt\":\"GT\"},{\"first\":\"name\",\"second\":\"abc\",\"opt\":\"EQ\"}]")
        )
    } // Fun goodFilterDataProvider()

    /**
     * 正确过滤条件测试
     *
     * @param   filter      过滤条件
     * @param   expected    预期结果
     */
    @Test(dataProvider="filter")
    fun goodFilterTest(filter: String, expected: String) {
//        val queryParser = SSyntaxParser(ByteArrayInputStream(filter.toByteArray()))
//        val filters  = queryParser.parserFilters()
//        Assert.assertEquals(filters.toJson(), expected)
        return
    } // Fun goodFilterTest()

} // Class SSyntaxParserTest
